local arcpro = {}
	
	
	local inst = mc.mcGetInstance()
	local msg = ""
	
	local m_iniName = "HiCON_THC_PARAM"
	
	--Paramater Name, Paramater Type, Register Path
	arcpro.thcParams = {
		{"Tool Number", "int", ""},
		{"Description", "string", "ArcPro/THC_ToolDescription"},
		{"Target Volts", "float", "HiCON/DRO_To_Ctrl_18"},
		--{"VoltageMultiplier", "float", "HiCON/DRO_To_Ctrl_17"},
		{"Deadband Volts", "float", "HiCON/DRO_To_Ctrl_16"},
		{"Adjust Speed (%)", "float", "HiCON/DRO_To_Ctrl_19"},
		{"FR AntiDive (%)", "float", "ArcPro/AntiDiveFeedrate"},
		--{"TV AntiDive", "float", "ArcPro/AntiDiveTipVolts"},
		{"TV Op Range (%)", "float", "HiCON/DRO_To_Ctrl_14"},
		{"TV Rec Range (%)", "float", "HiCON/DRO_To_Ctrl_13"},
		--{"OmitTouchoff", "bool", "ArcPro/OmitTouchoff"},
		--{"MaxProbeDistance", "float", "ArcPro/MaxProbeDistance"},
		--{"ProbeOffset", "float", "ArcPro/ProbeOffset"},
		--{"ProbeSpeed", "float", "ArcPro/TouchSpeed"},
		{"Torch On Height", "float", "ArcPro/TorchOnHeight"},
		{"ArcOK Timeout (ms)", "int", "ArcPro/ArcOKDelay"},
		{"Pierce Height", "float", "ArcPro/PierceHeight"},
		{"Pierce Delay (ms)", "int", "ArcPro/PierceDelay"},
		{"Cut Height", "float", "ArcPro/THC_CutHeight"},
		{"Max Cut Height", "float", "ArcPro/MaxCutHeight"},
		{"Min Cut Height", "float", "ArcPro/MinCutHeight"},
		{"THC Delay (ms)", "int", "ArcPro/THC_Delay"}
		--{"RetractHeight", "float", "ArcPro/THC_RetractHeight"}
	}
	
	function arcpro.debugMessage(msg)
		local hDebugReg = mc.mcRegGetHandle(inst, "HiCON/DebugPrint")
		mc.mcRegSetValueString( hDebugReg, msg )
	end
	
	function arcpro.throwError(msg)
		mc.mcSpindleSetDirection(inst, mc.MC_SPINDLE_OFF)
		local hDebugReg = mc.mcRegGetHandle(inst, "HiCON/DebugPrint")
		mc.mcRegSetValueString( hDebugReg, "Error - " .. msg )
		mc.mcCntlMacroAlarm(inst, 3, msg)
	end

	
	function arcpro.loadSettingsTable()
	
		local settingsTable = {}
		
		local toolString = mc.mcProfileGetString(inst, "THCToolNumbers", "ToolNumbers", "0")
		
		--Converts string into table of numbers (ToolNumbers)
		--I.E. "0 1 5" -> {0, 1, 5}
		--This is to keep track of what tools are in use, since
		--mcProfile has no convenient way of compiling a list of
		--existing sections.
		local t = {}
		for n in string.gmatch(toolString, '%-?%d+') do
			local num = tonumber(n)
			if (num < 100000 and num > -100000) then
				t[#t+1] = tonumber(n) 
			end
		end
		table.sort(t)
		
		
		--Read in values from ini file
		for i, v in pairs(t) do
			local entry = {}
			for i = 1, #arcpro.thcParams do
				entry[i] = mc.mcProfileGetString(inst, m_iniName .. v, arcpro.thcParams[i][1], "")
			end
			
			settingsTable[#settingsTable + 1] = entry
		end
		
		return settingsTable
		
	end
	
	function arcpro.saveSettingsTable(settingsTable)
	
		local toolString = mc.mcProfileGetString(inst, "THCToolNumbers", "ToolNumbers", "0")
		
		--Converts string into table of numbers (ToolNumbers)
		--I.E. "0 1 5" -> {0, 1, 5}
		--This is to keep track of what tools are in use, since
		--mcProfile has no convenient way of compiling a list of
		--existing sections.
		local t = {}
		toolString:gsub('%-?%d+', function(n) t[#t+1] = tonumber(n) end)
		
		for i, v in pairs(t) do
			mc.mcProfileDeleteSection(inst, m_iniName .. v)
		end
		
		toolString = ""
		
		local entry = {}
		
		for i = 1, #settingsTable do
			for j = 1, #arcpro.thcParams do
				mc.mcProfileWriteString(inst, m_iniName .. settingsTable[i][1], arcpro.thcParams[j][1], tostring(settingsTable[i][j]))
			end
			
			toolString = toolString .. " " .. settingsTable[i][1]
			
		end
		
		mc.mcProfileWriteString(inst, "THCToolNumbers", "ToolNumbers", toolString)
		
	end
	
	function arcpro.getCurrentSettings()
		
		settings = {}
		
		settings[1] = mc.mcToolGetCurrent(inst)
		settings[1] = tostring(math.tointeger(settings[1]))
		
		
		for i = 2, #arcpro.thcParams do
			if (arcrp.thcParams[i][3] ~= "") then
				local hReg = mc.mcRegGetHandle(inst, arcpro.thcParams[i][3])
				settings[i] = mc.mcRegGetValueString(hReg)
			else
				settings[i] = ""
			end
		end
		return settings
	end
	
	function arcpro.applySettings(settings)
		
		for i, entry in pairs(arcpro.thcParams) do
			if (entry[3] ~= "") then
				local hReg = mc.mcRegGetHandle(inst, entry[3])
				mc.mcRegSetValueString(hReg, settings[i])
			end
		end
	
	end
	
	function arcpro.thcEnable(Enabled)	
		local motionDevice = mc.mcProfileGetString(inst, "Preferences", "MotionDevice", 'NO MOTION')
		local commandReg = mc.mcRegGetHandle(inst, motionDevice .. "/Command")
		
		local hReg = mc.mcRegGetHandle(inst, "HiCON/DRO_To_Ctrl_15")
		
		if (Enabled == 1) then
			mc.mcRegSetValue(hReg, 99000)
			mc.mcRegSendCommand(commandReg, "ENABLE_THC_MOTION")
		else
			mc.mcRegSetValue(hReg, 0)
			mc.mcRegSendCommand(commandReg, "DISABLE_THC_MOTION")
		end
		
	end
	
	
return arcpro
